package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.tags.ItemTags
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraftforge.common.Tags
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.recipe.UpgradeRecipe

fun addCraftingTableRecipes(consumer: RecipeOutput) {
	MatteryRecipe(MRegistry.CARGO_CRATES.item)
		.row(MItemTags.TRITANIUM_PLATES, multiIngredient(Tags.Items.CHESTS_WOODEN, Tags.Items.BARRELS_WOODEN), MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(Tags.Items.CHESTS)
		.build(consumer)

	for ((dye, crate) in MRegistry.CARGO_CRATES.blocks) {
		ShapelessRecipeBuilder(crate, 1)
			.requires(Ingredient.of(MRegistry.CARGO_CRATES.allItems.entries.stream().filter { it.key != dye }.map { ItemStack(it.value) }))
			.requires(dye.tag)
			.unlockedBy(MRegistry.CARGO_CRATES.allItems.entries.stream().filter { it.key != dye }.map { it.value })
			.save(consumer, "${crate.registryName}_alt")
	}

	ShapelessRecipeBuilder(MItems.TRITANIUM_INGOT_BLOCK, 1)
		.requires(Ingredient.of(MItemTags.TRITANIUM_INGOTS), 9)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.save(consumer)

	ShapelessRecipeBuilder(MItems.TRITANIUM_INGOT, 9)
		.requires(Ingredient.of(MItemTags.TRITANIUM_INGOTS_STORAGE))
		.unlockedBy(MItemTags.TRITANIUM_INGOTS_STORAGE)
		.save(consumer, modLocation("tritanium_ingot_from_storage"))

	MItems.ENERGY_COUNTER.values.forEach {
		ShapelessRecipeBuilder(it, 1)
			.requires(it)
			.unlockedBy(it)
			.save(consumer, modLocation("${it.registryName!!.path}_reset"))
	}

	ShapelessRecipeBuilder(MItems.HOLO_SIGN, 1)
		.requires(MItems.HOLO_SIGN)
		.unlockedBy(MItems.HOLO_SIGN)
		.save(consumer, modLocation("holo_sign_reset"))

	MatteryRecipe(MBlocks.ENERGY_COUNTER[null]!!)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ENERGY_BUS, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.BASIC_CIRCUIT, Tags.Items.DUSTS_GLOWSTONE, MItemTags.HARDENED_GLASS_PANES_COLORLESS)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ENERGY_BUS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MBlocks.TWIN_PLATE_PRESS[null]!!)
		.rowAC(MItems.ELECTROMOTOR, MItems.ELECTROMOTOR)
		.row(MItemTags.PISTONS, MItems.MACHINE_FRAME, MItemTags.PISTONS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItems.MACHINE_FRAME)
		.build(consumer)

	for ((color, item) in MItems.PLATE_PRESS) {
		MatteryRecipe(MBlocks.TWIN_PLATE_PRESS[color]!!)
			.setUpgradeSource(item)
			.addUpgradeOps(UpgradeRecipe.All)
			.rowA(item)
			.build(consumer, "plate_press_migration/${color?.name?.lowercase() ?: "default"}")
	}

	MatteryRecipe(MBlocks.CHEMICAL_GENERATOR[null]!!)
		.rowB(MItems.ENERGY_BUS)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.FURNACES, MItemTags.TRITANIUM_INGOTS)
		.rowAC(MItems.ELECTRIC_PARTS, MItems.ELECTRIC_PARTS)
		.unlockedBy(MItemTags.FURNACES)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.PATTERN_DRIVE_NORMAL)
		.rowAC(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.row(MItemTags.ADVANCED_CIRCUIT, MItemTags.TRITANIUM_PLATES, MItemTags.ADVANCED_CIRCUIT)
		.rowAC(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.unlockedBy(MItemTags.ADVANCED_CIRCUIT)
		.build(consumer)

	// Машины
	MatteryRecipe(MItems.MATTER_RECYCLER[null]!!)
		.row(MItems.MATTER_CAPACITOR_PARTS, Items.HOPPER, MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItems.MATTER_CABLE, MItems.MATTER_IO_PORT, MItems.MATTER_CABLE)
		.unlockedBy(MItems.MATTER_CABLE)
		.build(consumer)

	// Блоки
	MatteryRecipe(MItems.MATTER_CAPACITOR_BANK[null]!!)
		.row(Tags.Items.GLASS, MItemTags.IRON_PLATES, Tags.Items.GLASS)
		.row(MItemTags.IRON_PLATES, MItems.MACHINE_FRAME, MItemTags.IRON_PLATES)
		.row(MItems.MATTER_CABLE, MItems.MATTER_IO_PORT, MItems.MATTER_CABLE)
		.unlockedBy(MItems.MATTER_CABLE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_BANK[null]!!)
		.row(Tags.Items.GLASS, MItemTags.IRON_PLATES, Tags.Items.GLASS)
		.row(MItemTags.IRON_PLATES, MItems.MACHINE_FRAME, MItemTags.IRON_PLATES)
		.row(MItems.ELECTRIC_PARTS, MItems.ENERGY_BUS, MItems.ELECTRIC_PARTS)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_STABILIZER)
		.row(MItemTags.ADVANCED_CIRCUIT, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.ADVANCED_CIRCUIT)
		.row(MItems.MATTER_TRANSFORM_MATRIX, MItems.MACHINE_FRAME, MItems.MATTER_TRANSFORM_MATRIX)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_LIMITER, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_LIMITER)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.PORTABLE_GRAVITATION_STABILIZER)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_LIMITER, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_LIMITER)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_FIELD_SENSOR)
		.rowB(MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTROMAGNET, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.IRON_PLATES)
		.unlockedBy(MItems.ELECTROMAGNET)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_FIELD_LIMITER)
		.row(Tags.Items.ENDER_PEARLS, MItemTags.ADVANCED_CIRCUIT, Tags.Items.ENDER_PEARLS)
		.row(MItemTags.GOLD_WIRES, MItems.QUANTUM_TRANSCEIVER, MItemTags.GOLD_WIRES)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.QUANTUM_TRANSCEIVER)
		.build(consumer)

	MatteryRecipe(MItems.BLACK_HOLE_SCANNER)
		.row(MItemTags.IRON_PLATES, Tags.Items.GLASS_PANES_COLORLESS, MItemTags.IRON_PLATES)
		.row(MItemTags.GOLD_WIRES, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.ADVANCED_CIRCUIT)
		.rowAC(Tags.Items.DUSTS_GLOWSTONE, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.PHANTOM_ATTRACTOR)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.GLASS_COLORLESS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.TRITANIUM_PLATES, MItems.QUANTUM_TRANSCEIVER, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, ItemTags.BEDS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.QUANTUM_TRANSCEIVER)
		.build(consumer)

	MatteryRecipe(MItems.QUANTUM_TRANSCEIVER, 2)
		.rowAC(MItemTags.COPPER_WIRES, MItemTags.COPPER_WIRES)
		.row(MItemTags.GOLD_WIRES, Tags.Items.ENDER_PEARLS, MItemTags.GOLD_WIRES)
		.row(MItemTags.GOLD_WIRES, MItems.ELECTROMAGNET, MItemTags.GOLD_WIRES)
		.unlockedBy(Tags.Items.ENDER_PEARLS)
		.build(consumer)

	MatteryRecipe(MItems.ELECTROMAGNET)
		.row(MItemTags.COPPER_WIRES, Tags.Items.INGOTS_IRON, MItemTags.COPPER_WIRES)
		.unlockedBy(Tags.Items.INGOTS_IRON)
		.build(consumer)

	MatteryRecipe(MItems.ELECTROMOTOR)
		.rowB(MItems.ELECTROMAGNET)
		.row(MItems.ELECTROMAGNET, Tags.Items.INGOTS_IRON, MItems.ELECTROMAGNET)
		.row(MItemTags.COPPER_WIRES, Tags.Items.INGOTS_IRON, MItemTags.COPPER_WIRES)
		.unlockedBy(MItems.ELECTROMAGNET)
		.build(consumer)

	MatteryRecipe(MItems.ENERGY_SERVO[null]!!)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ENERGY_BUS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.MIRROR_COMPOUND, 3)
		.row(Tags.Items.GLASS_PANES_COLORLESS, Tags.Items.GLASS_PANES_COLORLESS, Tags.Items.GLASS_PANES_COLORLESS)
		.row(MItemTags.IRON_PLATES, MItemTags.IRON_PLATES, MItemTags.IRON_PLATES)
		.unlockedBy(MItemTags.IRON_PLATES)
		.unlockedBy(Tags.Items.GLASS_PANES_COLORLESS)
		.build(consumer)

	MatteryRecipe(MItems.REINFORCED_TRITANIUM_PLATE)
			.rowB(MItemTags.CARBON_PLATES)
			.row(MItemTags.CARBON_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.CARBON_PLATES)
			.rowB(MItemTags.CARBON_PLATES)
			.unlockedBy(MItemTags.TRITANIUM_PLATES)
			.build(consumer)

	MatteryRecipe(MItems.CARBON_FIBRE_BLOCK)
		.rowAB(MItemTags.CARBON_PLATES, MItemTags.CARBON_PLATES)
		.rowAB(MItemTags.CARBON_PLATES, MItemTags.CARBON_PLATES)
		.unlockedBy(MItemTags.CARBON_PLATES)
		.build(consumer)

	ShapelessRecipeBuilder(MItems.CARBON_MESH, 4)
		.requires(Ingredient.of(MItems.CARBON_FIBRE_BLOCK))
		.unlockedBy(MItemTags.CARBON_PLATES)
		.unlockedBy(MItems.CARBON_FIBRE_BLOCK)
		.save(consumer, modLocation("carbon_mesh_from_block"))

	// броня
	MatteryRecipe(MItems.TRITANIUM_HELMET)
		.setUpgradeSource(Items.LEATHER_HELMET)
		.addUpgradeOps(
			UpgradeRecipe.Direct("display"),
			UpgradeRecipe.Direct("Enchantments"),
		)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, Items.LEATHER_HELMET, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.unlockedBy(MItemTags.REINFORCED_TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_PANTS)
		.setUpgradeSource(Items.LEATHER_LEGGINGS)
		.addUpgradeOps(
			UpgradeRecipe.Direct("display"),
			UpgradeRecipe.Direct("Enchantments"),
		)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, Items.LEATHER_LEGGINGS, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.rowAC(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.unlockedBy(MItemTags.REINFORCED_TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_CHESTPLATE)
		.setUpgradeSource(Items.LEATHER_CHESTPLATE)
		.addUpgradeOps(
			UpgradeRecipe.Direct("display"),
			UpgradeRecipe.Direct("Enchantments"),
		)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, Items.LEATHER_CHESTPLATE, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.unlockedBy(MItemTags.REINFORCED_TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_BOOTS)
		.setUpgradeSource(Items.LEATHER_BOOTS)
		.addUpgradeOps(
			UpgradeRecipe.Direct("display"),
			UpgradeRecipe.Direct("Enchantments"),
		)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, Items.LEATHER_BOOTS, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.rowAC(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.unlockedBy(MItemTags.REINFORCED_TRITANIUM_PLATES)
		.build(consumer)

	// простая броня
	MatteryRecipe(MItems.SIMPLE_TRITANIUM_HELMET)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.rowAC(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.build(consumer)

	MatteryRecipe(MItems.SIMPLE_TRITANIUM_PANTS)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.rowAC(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.rowAC(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.build(consumer)

	MatteryRecipe(MItems.SIMPLE_TRITANIUM_CHESTPLATE)
		.rowAC(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.build(consumer)

	MatteryRecipe(MItems.SIMPLE_TRITANIUM_BOOTS)
		.rowAC(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.rowAC(MItemTags.TRITANIUM_INGOTS, MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.build(consumer)

	// станция андроида
	MatteryRecipe(MItems.ANDROID_STATION[null]!!)
		.row(MItems.ELECTRIC_PARTS, MItems.QUANTUM_TRANSCEIVER, MItems.ELECTRIC_PARTS)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTRIC_PARTS, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// беспроводной зарядник андроидов
	MatteryRecipe(MItems.ANDROID_CHARGER[null]!!)
		.row(MItems.ELECTRIC_PARTS, MItems.QUANTUM_TRANSCEIVER, MItems.ELECTRIC_PARTS)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// апгрейд на сетку крафта
	MatteryRecipe(MItems.ExopackUpgrades.CRAFTING_UPGRADE)
		.row(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.row(MItemTags.CRAFTING_TABLES, MItems.QUANTUM_TRANSCEIVER, MItemTags.CRAFTING_TABLES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// апгрейд на переплавку
	MatteryRecipe(MItems.ExopackUpgrades.SMELTING_UPGRADE)
		.row(MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT, MItemTags.ADVANCED_CIRCUIT)
		.row(Items.FURNACE, MItems.QUANTUM_TRANSCEIVER, Items.FURNACE)
		.row(MItemTags.TRITANIUM_PLATES, Items.FURNACE, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// апгрейд на эндер сундук
	MatteryRecipe(MItems.ExopackUpgrades.ENDER_UPGRADE)
		.row(MItemTags.ADVANCED_CIRCUIT, MItems.ELECTROMAGNET, MItemTags.ADVANCED_CIRCUIT)
		.row(MItems.ELECTROMAGNET, Items.ENDER_CHEST, MItems.ELECTROMAGNET)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTROMAGNET, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// генератор коблы
	MatteryRecipe(MItems.COBBLESTONE_GENERATOR[null]!!)
		.row(MItemTags.HARDENED_GLASS_COLORLESS, MItems.TRITANIUM_PICKAXE, MItemTags.HARDENED_GLASS_COLORLESS)
		.row(Items.LAVA_BUCKET, Items.HOPPER, Items.WATER_BUCKET)
		.rowB(Tags.Items.CHESTS)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_SHEARS)
		.rowB(MItemTags.TRITANIUM_INGOTS)
		.rowA(MItemTags.TRITANIUM_INGOTS)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_SHIELD)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, Items.SHIELD, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.row(MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES, MItemTags.REINFORCED_TRITANIUM_PLATES)
		.rowB(MItemTags.REINFORCED_TRITANIUM_PLATES)
		.build(consumer)

	ShapelessRecipeBuilder(MItems.TRITANIUM_NUGGET, 9)
		.requires(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_NUGGETS)
		.save(consumer)

	ShapelessRecipeBuilder(MItems.TRITANIUM_INGOT, 1)
		.requires(Ingredient.of(MItemTags.TRITANIUM_NUGGETS), 9)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.TRITANIUM_NUGGETS)
		.save(consumer, modLocation("ingot_from_nuggets"))

	MatteryRecipe(MItems.ESSENCE_STORAGE[null]!!)
		.row(MItems.MATTER_CAPACITOR_PARTS, Items.ENDER_EYE, MItemTags.ADVANCED_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.GOLD_WIRES, MItemTags.HARDENED_GLASS, MItemTags.HARDENED_GLASS)
		.build(consumer)

	MatteryRecipe(MItems.ESSENCE_SERVO)
		.row(MItemTags.TRITANIUM_PLATES, MItems.QUANTUM_TRANSCEIVER, MItemTags.TRITANIUM_PLATES)
		.rowB(Tags.Items.RODS_WOODEN)
		.rowB(Tags.Items.RODS_WOODEN)
		.build(consumer)

	for ((dye, item) in MItems.MATTER_REPLICATOR) {
		MatteryRecipe(MItems.MATTER_RECONSTRUCTOR[dye]!!)
			.setUpgradeSource(item)
			.addUpgradeOps(
				UpgradeRecipe.Indirect("BlockEntityTag.${MatteryBlockEntity.ENERGY_KEY}", "BlockEntityTag.energy"),
				UpgradeRecipe.Indirect("BlockEntityTag.${MatteryBlockEntity.MATTER_STORAGE_KEY}", "BlockEntityTag.matter"),
			)
			.row(MItemTags.ADVANCED_CIRCUIT, Tags.Items.GEMS_EMERALD, MItemTags.ADVANCED_CIRCUIT)
			.row(MItems.ELECTRIC_PARTS, item, MItems.ELECTRIC_PARTS)
			.row(MItems.ELECTROMAGNET, MItems.ELECTROMAGNET, MItems.ELECTROMAGNET)
			.build(consumer)
	}

	MatteryRecipe(MItems.MATTER_SCANNER[null]!!)
		.row(MItems.MIRROR, MItems.MATTER_TRANSFORM_MATRIX, MItems.MIRROR)
		.row(MItemTags.HARDENED_GLASS_COLORLESS, MItems.MACHINE_FRAME, MItemTags.HARDENED_GLASS_COLORLESS)
		.row(MItems.MATTER_CABLE, MItemTags.ADVANCED_CIRCUIT, MItems.MATTER_CABLE)
		.build(consumer)

	MatteryRecipe(MItems.FLUID_CAPSULE, count = 8)
		.row(MItemTags.TRITANIUM_NUGGETS, MItemTags.TRITANIUM_NUGGETS, MItemTags.TRITANIUM_NUGGETS)
		.rowB(MItemTags.HARDENED_GLASS_PANES)
		.row(MItemTags.TRITANIUM_NUGGETS, MItemTags.TRITANIUM_NUGGETS, MItemTags.TRITANIUM_NUGGETS)
		.unlockedBy(MItemTags.HARDENED_GLASS_PANES)
		.unlockedBy(MItemTags.TRITANIUM_NUGGETS)
		.build(consumer)

	MatteryRecipe(MItems.FLUID_TANK)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.HARDENED_GLASS_PANES, MItemTags.TRITANIUM_INGOTS)
		.rowAC(MItemTags.HARDENED_GLASS_PANES, MItemTags.HARDENED_GLASS_PANES)
		.row(MItemTags.TRITANIUM_INGOTS, MItemTags.HARDENED_GLASS_PANES, MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItemTags.HARDENED_GLASS_PANES)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.build(consumer)

	MatteryRecipe(MItems.EXPLOSIVE_HAMMER)
		.rowB(Tags.Items.INGOTS_IRON)
		.rowAB(Tags.Items.INGOTS_IRON, Tags.Items.RODS_WOODEN)
		.rowB(Tags.Items.RODS_WOODEN)
		.unlockedBy(Items.FLINT_AND_STEEL)
		.build(consumer)

	MatteryRecipe(MItems.POWERED_FURNACE[null]!!)
		.row(Items.FURNACE, MItems.MACHINE_FRAME, Items.FURNACE)
		.unlockedBy(MItems.MACHINE_FRAME)
		.build(consumer)

	MatteryRecipe(MItems.POWERED_SMOKER[null]!!)
		.rowAC(Items.FURNACE, Items.FURNACE)
		.row(MItems.ELECTROMAGNET, MItems.MACHINE_FRAME, MItems.ELECTROMAGNET)
		.unlockedBy(MItems.MACHINE_FRAME)
		.build(consumer)

	MatteryRecipe(MItems.POWERED_BLAST_FURNACE[null]!!)
		.row(MItems.ELECTROMAGNET, Items.FURNACE, MItems.ELECTROMAGNET)
		.row(MItems.ELECTROMAGNET, MItems.MACHINE_FRAME, MItems.ELECTROMAGNET)
		.row(MItems.ELECTROMAGNET, Items.FURNACE, MItems.ELECTROMAGNET)
		.unlockedBy(MItems.MACHINE_FRAME)
		.build(consumer)

	MatteryRecipe(MItems.INFINITE_WATER_SOURCE)
		.row(MItemTags.IRON_PLATES, MItemTags.IRON_PLATES, MItemTags.IRON_PLATES)
		.rowAC(Items.WATER_BUCKET, Items.WATER_BUCKET)
		.row(MItemTags.IRON_PLATES, MItemTags.IRON_PLATES, MItemTags.IRON_PLATES)
		.unlockedBy(Items.WATER_BUCKET)
		.build(consumer)

	MatteryRecipe(MItems.PAINTER)
		.row(Tags.Items.RODS_WOODEN, Items.BUCKET, Items.BUCKET)
		.row(MItemTags.IRON_PLATES, Items.BUCKET, MItemTags.IRON_PLATES)
		.row(MItemTags.IRON_PLATES, MItemTags.CRAFTING_TABLES, MItemTags.IRON_PLATES)
		.unlockedBy(Tags.Items.DYES)
		.build(consumer)
}
